@@ -431,8 +431,8 @@ static zend_always_inline zend_string *accel_find_interned_string(zend_string *s
 	}
 
 	if (!ZCG(counted)) {
-		if (accel_activate_add() == FAILURE) {
-			return str;
+		if (!ZCG(accelerator_enabled) || accel_activate_add() == FAILURE) {
+			return NULL;
 		}
 		ZCG(counted) = 1;
 	}
@@ -572,10 +572,7 @@ static void accel_copy_permanent_strings(zend_new_interned_string_func_t new_int
 	/* empty string */
 	zend_empty_string = new_interned_string(zend_empty_string);
 	for (j = 0; j < 256; j++) {
-		char s[2];
-		s[0] = j;
-		s[1] = 0;
-		zend_one_char_string[j] = new_interned_string(zend_string_init(s, 1, 0));
+		zend_one_char_string[j] = new_interned_string(ZSTR_CHAR(j));
 	}
 	for (j = 0; j < ZEND_STR_LAST_KNOWN; j++) {
 		zend_known_strings[j] = new_interned_string(zend_known_strings[j]);
@@ -739,12 +736,11 @@ static zend_string* ZEND_FASTCALL accel_replace_string_by_shm_permanent(zend_str
 static zend_string* ZEND_FASTCALL accel_replace_string_by_process_permanent(zend_string *str)
 {
 	zend_string *ret = zend_interned_string_find_permanent(str);
-
 	if (ret) {
 		zend_string_release(str);
 		return ret;
 	}
-	ZEND_ASSERT(0);
+	ZEND_ASSERT(!IS_ACCEL_INTERNED(str));
 	return str;
 }
 
@@ -758,10 +754,9 @@ static void accel_use_shm_interned_strings(void)
 	if (ZCSG(interned_strings).saved_top == NULL) {
 		accel_copy_permanent_strings(accel_new_interned_string);
 	} else {
+		ZCG(counted) = 1;
 		accel_copy_permanent_strings(accel_replace_string_by_shm_permanent);
-		if (ZCG(counted)) {
-			accel_deactivate_sub();
-		}
+		ZCG(counted) = 0;
 	}
 	accel_interned_strings_save_state();
 
@@ -1168,7 +1163,7 @@ char *accel_make_persistent_key(const char *path, size_t path_length, int *key_l
 			cwd_len = ZSTR_LEN(cwd_str);
 			if (ZCG(cwd_check)) {
 				ZCG(cwd_check) = 0;
-				if ((ZCG(counted) || ZCSG(accelerator_enabled))) {
+				if (ZCG(accelerator_enabled)) {
 
 					zend_string *str = accel_find_interned_string(cwd_str);
 					if (!str) {
@@ -1191,7 +1186,11 @@ char *accel_make_persistent_key(const char *path, size_t path_length, int *key_l
 						cwd_len = ZCG(cwd_key_len) = buf + sizeof(buf) - 1 - res;
 						cwd = ZCG(cwd_key);
 						memcpy(ZCG(cwd_key), res, cwd_len + 1);
+					} else {
+						return NULL;
 					}
+				} else {
+					return NULL;
 				}
 			}
 		}
@@ -1208,7 +1207,7 @@ char *accel_make_persistent_key(const char *path, size_t path_length, int *key_l
 
 			if (ZCG(include_path_check)) {
 				ZCG(include_path_check) = 0;
-				if ((ZCG(counted) || ZCSG(accelerator_enabled))) {
+				if (ZCG(accelerator_enabled)) {
 
 					zend_string *str = accel_find_interned_string(ZCG(include_path));
 					if (!str) {
@@ -1230,7 +1229,11 @@ char *accel_make_persistent_key(const char *path, size_t path_length, int *key_l
 						include_path_len = ZCG(include_path_key_len) = buf + sizeof(buf) - 1 - res;
 						include_path = ZCG(include_path_key);
 						memcpy(ZCG(include_path_key), res, include_path_len + 1);
+					} else {
+						return NULL;
 					}
+				} else {
+					return NULL;
 				}
 			}
 		}
@@ -1291,7 +1294,7 @@ int zend_accel_invalidate(const char *filename, size_t filename_len, zend_bool f
 	zend_string *realpath;
 	zend_persistent_script *persistent_script;
 
-	if (!ZCG(enabled) || !accel_startup_ok || !ZCSG(accelerator_enabled) || accelerator_shm_read_lock() != SUCCESS) {
+	if (!ZCG(accelerator_enabled) || accelerator_shm_read_lock() != SUCCESS) {
 		return FAILURE;
 	}
 
@@ -1387,6 +1390,10 @@ static zend_persistent_script *store_script_in_file_cache(zend_persistent_script
 	/* Align to 64-byte boundary */
 	ZCG(mem) = zend_arena_alloc(&CG(arena), memory_used + 64);
 	ZCG(mem) = (void*)(((zend_uintptr_t)ZCG(mem) + 63L) & ~63L);
+#elif ZEND_MM_ALIGNMENT < 8
+	/* Align to 8-byte boundary */
+	ZCG(mem) = zend_arena_alloc(&CG(arena), memory_used + 8);
+	ZCG(mem) = (void*)(((zend_uintptr_t)ZCG(mem) + 7L) & ~7L);
 #else
 	ZCG(mem) = zend_arena_alloc(&CG(arena), memory_used);
 #endif
@@ -1907,20 +1914,33 @@ zend_op_array *persistent_compile_file(zend_file_handle *file_handle, int type)
 	int key_length;
 	int from_shared_memory; /* if the script we've got is stored in SHM */
 
-	if (!file_handle->filename || !ZCG(enabled) || !accel_startup_ok) {
+	if (!file_handle->filename || !ZCG(accelerator_enabled)) {
 		/* The Accelerator is disabled, act as if without the Accelerator */
+		ZCG(cache_opline) = NULL;
+		ZCG(cache_persistent_script) = NULL;
+#ifdef HAVE_OPCACHE_FILE_CACHE
+		if (file_handle->filename
+		 && ZCG(accel_directives).file_cache
+		 && ZCG(enabled) && accel_startup_ok) {
+			return file_cache_compile_file(file_handle, type);
+		}
+#endif
 		return accelerator_orig_compile_file(file_handle, type);
 #ifdef HAVE_OPCACHE_FILE_CACHE
 	} else if (file_cache_only) {
+		ZCG(cache_opline) = NULL;
+		ZCG(cache_persistent_script) = NULL;
 		return file_cache_compile_file(file_handle, type);
 #endif
-	} else if ((!ZCG(counted) && !ZCSG(accelerator_enabled)) ||
+	} else if (!ZCG(accelerator_enabled) ||
 	           (ZCSG(restart_in_progress) && accel_restart_is_active())) {
 #ifdef HAVE_OPCACHE_FILE_CACHE
 		if (ZCG(accel_directives).file_cache) {
 			return file_cache_compile_file(file_handle, type);
 		}
 #endif
+		ZCG(cache_opline) = NULL;
+		ZCG(cache_persistent_script) = NULL;
 		return accelerator_orig_compile_file(file_handle, type);
 	}
 
@@ -1947,10 +1967,14 @@ zend_op_array *persistent_compile_file(zend_file_handle *file_handle, int type)
 			/* try to find cached script by key */
 			key = accel_make_persistent_key(file_handle->filename, strlen(file_handle->filename), &key_length);
 			if (!key) {
+				ZCG(cache_opline) = NULL;
+				ZCG(cache_persistent_script) = NULL;
 				return accelerator_orig_compile_file(file_handle, type);
 			}
 			persistent_script = zend_accel_hash_str_find(&ZCSG(hash), key, key_length);
 		} else if (UNEXPECTED(is_stream_path(file_handle->filename) && !is_cacheable_stream_path(file_handle->filename))) {
+			ZCG(cache_opline) = NULL;
+			ZCG(cache_persistent_script) = NULL;
 			return accelerator_orig_compile_file(file_handle, type);
 		}
 
@@ -2209,12 +2233,11 @@ static int persistent_stream_open_function(const char *filename, zend_file_handl
 /* zend_resolve_path() replacement for PHP 5.3 and above */
 static zend_string* persistent_zend_resolve_path(const char *filename, size_t filename_len)
 {
-	if (ZCG(enabled) && accel_startup_ok &&
+	if (
 #ifdef HAVE_OPCACHE_FILE_CACHE
 		!file_cache_only &&
 #endif
-	    (ZCG(counted) || ZCSG(accelerator_enabled)) &&
-	    !ZCSG(restart_in_progress)) {
+	    ZCG(accelerator_enabled)) {
 
 		/* check if callback is called from include_once or it's a main request */
 		if ((!EG(current_execute_data) &&
@@ -2315,9 +2338,8 @@ static void accel_reset_pcre_cache(void)
 
 static void accel_activate(void)
 {
-	zend_bool reset_pcre = 0;
-
 	if (!ZCG(enabled) || !accel_startup_ok) {
+		ZCG(accelerator_enabled) = 0;
 		return;
 	}
 
@@ -2345,6 +2367,7 @@ static void accel_activate(void)
 
 #ifdef HAVE_OPCACHE_FILE_CACHE
 	if (file_cache_only) {
+		ZCG(accelerator_enabled) = 0;
 		return;
 	}
 #endif
@@ -2420,16 +2443,16 @@ static void accel_activate(void)
 				}
 				accel_restart_leave();
 			}
-		} else {
-			reset_pcre = 1;
 		}
 		zend_shared_alloc_unlock();
 	}
 
+	ZCG(accelerator_enabled) = ZCSG(accelerator_enabled);
+
 	SHM_PROTECT();
 	HANDLE_UNBLOCK_INTERRUPTIONS();
 
-	if (ZCSG(last_restart_time) != ZCG(last_restart_time)) {
+	if (ZCG(accelerator_enabled) && ZCSG(last_restart_time) != ZCG(last_restart_time)) {
 		/* SHM was reinitialized. */
 		ZCG(last_restart_time) = ZCSG(last_restart_time);
 
@@ -2437,8 +2460,10 @@ static void accel_activate(void)
 		realpath_cache_clean();
 
 		accel_reset_pcre_cache();
-	} else if (reset_pcre) {
+		ZCG(pcre_reseted) = 0;
+	} else if (!ZCG(accelerator_enabled) && !ZCG(pcre_reseted)) {
 		accel_reset_pcre_cache();
+		ZCG(pcre_reseted) = 1;
 	}
 }
 
@@ -2466,10 +2491,6 @@ static void accel_deactivate(void)
 		zend_string_release_ex(ZCG(cwd), 0);
 		ZCG(cwd) = NULL;
 	}
-
-	if (!ZCG(enabled) || !accel_startup_ok) {
-		return;
-	}
 }
 
 static int accelerator_remove_cb(zend_extension *element1, zend_extension *element2)
@@ -2511,6 +2532,7 @@ static inline int accel_find_sapi(void)
 		"cli-server",
 		"cgi-fcgi",
 		"fpm-fcgi",
+		"fpmi-fcgi",
 		"apache2handler",
 		"litespeed",
 		"uwsgi",
