@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2016 The PHP Group                                |
+   | Copyright (c) 1997-2018 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -13,13 +13,11 @@
    | license@php.net so we can mail you a copy immediately.               |
    +----------------------------------------------------------------------+
    | Authors: Rasmus Lerdorf <rasmus@php.net>                             |
-   |          Zeev Suraski <zeev@zend.com>                                |
+   |          Zeev Suraski <zeev@php.net>                                 |
    |          Colin Viebrock <colin@viebrock.ca>                          |
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #include "php.h"
 #include "php_ini.h"
 #include "php_globals.h"
@@ -37,16 +35,11 @@
 #ifdef HAVE_SYS_UTSNAME_H
 #include <sys/utsname.h>
 #endif
-
+#include "url.h"
+#include "php_string.h"
 
 #ifdef PHP_WIN32
-typedef void (WINAPI *PGNSI)(LPSYSTEM_INFO);
-typedef BOOL (WINAPI *PGPI)(DWORD, DWORD, DWORD, DWORD, PDWORD);
 # include "winver.h"
-
-# if _MSC_VER < 1300
-#  define OSVERSIONINFOEX php_win_OSVERSIONINFOEX
-# endif
 #endif
 
 #define SECTION(name)	if (!sapi_module.phpinfo_as_text) { \
@@ -61,16 +54,14 @@ PHPAPI extern char *php_ini_opened_path;
 PHPAPI extern char *php_ini_scanned_path;
 PHPAPI extern char *php_ini_scanned_files;
 
-static int php_info_print_html_esc(const char *str, int len) /* {{{ */
+static int php_info_print_html_esc(const char *str, size_t len) /* {{{ */
 {
-	size_t new_len;
-	int written;
-	char *new_str;
-	TSRMLS_FETCH();
+	size_t written;
+	zend_string *new_str;
 
-	new_str = php_escape_html_entities((unsigned char *) str, len, &new_len, 0, ENT_QUOTES, "utf-8" TSRMLS_CC);
-	written = php_output_write(new_str, new_len TSRMLS_CC);
-	str_efree(new_str);
+	new_str = php_escape_html_entities((unsigned char *) str, len, 0, ENT_QUOTES, "utf-8");
+	written = php_output_write(ZSTR_VAL(new_str), ZSTR_LEN(new_str));
+	zend_string_free(new_str);
 	return written;
 }
 /* }}} */
@@ -78,15 +69,14 @@ static int php_info_print_html_esc(const char *str, int len) /* {{{ */
 static int php_info_printf(const char *fmt, ...) /* {{{ */
 {
 	char *buf;
-	int len, written;
+	size_t len, written;
 	va_list argv;
-	TSRMLS_FETCH();
 
 	va_start(argv, fmt);
 	len = vspprintf(&buf, 0, fmt, argv);
 	va_end(argv);
 
-	written = php_output_write(buf, len TSRMLS_CC);
+	written = php_output_write(buf, len);
 	efree(buf);
 	return written;
 }
@@ -94,19 +84,17 @@ static int php_info_printf(const char *fmt, ...) /* {{{ */
 
 static int php_info_print(const char *str) /* {{{ */
 {
-	TSRMLS_FETCH();
-	return php_output_write(str, strlen(str) TSRMLS_CC);
+	return php_output_write(str, strlen(str));
 }
 /* }}} */
 
-static void php_info_print_stream_hash(const char *name, HashTable *ht TSRMLS_DC) /* {{{ */
+static void php_info_print_stream_hash(const char *name, HashTable *ht) /* {{{ */
 {
-	char *key;
-	uint len;
+	zend_string *key;
 
 	if (ht) {
 		if (zend_hash_num_elements(ht)) {
-			HashPosition pos;
+			int first = 1;
 
 			if (!sapi_module.phpinfo_as_text) {
 				php_info_printf("<tr><td class=\"e\">Registered %s</td><td class=\"v\">", name);
@@ -114,21 +102,20 @@ static void php_info_print_stream_hash(const char *name, HashTable *ht TSRMLS_DC
 				php_info_printf("\nRegistered %s => ", name);
 			}
 
-			zend_hash_internal_pointer_reset_ex(ht, &pos);
-			while (zend_hash_get_current_key_ex(ht, &key, &len, NULL, 0, &pos) == HASH_KEY_IS_STRING)
-			{
-				if (!sapi_module.phpinfo_as_text) {
-					php_info_print_html_esc(key, len-1);
-				} else {
-					php_info_print(key);
+			ZEND_HASH_FOREACH_STR_KEY(ht, key) {
+				if (key) {
+					if (first) {
+						first = 0;
+					} else {
+						php_info_print(", ");
+					}
+					if (!sapi_module.phpinfo_as_text) {
+						php_info_print_html_esc(ZSTR_VAL(key), ZSTR_LEN(key));
+					} else {
+						php_info_print(ZSTR_VAL(key));
+					}
 				}
-				zend_hash_move_forward_ex(ht, &pos);
-				if (zend_hash_get_current_key_ex(ht, &key, &len, NULL, 0, &pos) == HASH_KEY_IS_STRING) {
-					php_info_print(", ");
-				} else {
-					break;
-				}
-			}
+			} ZEND_HASH_FOREACH_END();
 
 			if (!sapi_module.phpinfo_as_text) {
 				php_info_print("</td></tr>\n");
@@ -144,18 +131,23 @@ static void php_info_print_stream_hash(const char *name, HashTable *ht TSRMLS_DC
 }
 /* }}} */
 
-PHPAPI void php_info_print_module(zend_module_entry *zend_module TSRMLS_DC) /* {{{ */
+PHPAPI void php_info_print_module(zend_module_entry *zend_module) /* {{{ */
 {
 	if (zend_module->info_func || zend_module->version) {
 		if (!sapi_module.phpinfo_as_text) {
-			php_info_printf("<h2><a name=\"module_%s\">%s</a></h2>\n", zend_module->name, zend_module->name);
+			zend_string *url_name = php_url_encode(zend_module->name, strlen(zend_module->name));
+
+			php_strtolower(ZSTR_VAL(url_name), ZSTR_LEN(url_name));
+			php_info_printf("<h2><a name=\"module_%s\">%s</a></h2>\n", ZSTR_VAL(url_name), zend_module->name);
+
+			efree(url_name);
 		} else {
 			php_info_print_table_start();
 			php_info_print_table_header(1, zend_module->name);
 			php_info_print_table_end();
 		}
 		if (zend_module->info_func) {
-			zend_module->info_func(zend_module TSRMLS_CC);
+			zend_module->info_func(zend_module);
 		} else {
 			php_info_print_table_start();
 			php_info_print_table_row(2, "Version", zend_module->version);
@@ -172,19 +164,21 @@ PHPAPI void php_info_print_module(zend_module_entry *zend_module TSRMLS_DC) /* {
 }
 /* }}} */
 
-static int _display_module_info_func(zend_module_entry *module TSRMLS_DC) /* {{{ */
+static int _display_module_info_func(zval *el) /* {{{ */
 {
+	zend_module_entry *module = (zend_module_entry*)Z_PTR_P(el);
 	if (module->info_func || module->version) {
-		php_info_print_module(module TSRMLS_CC);
+		php_info_print_module(module);
 	}
 	return ZEND_HASH_APPLY_KEEP;
 }
 /* }}} */
 
-static int _display_module_info_def(zend_module_entry *module TSRMLS_DC) /* {{{ */
+static int _display_module_info_def(zval *el) /* {{{ */
 {
+	zend_module_entry *module = (zend_module_entry*)Z_PTR_P(el);
 	if (!module->info_func && !module->version) {
-		php_info_print_module(module TSRMLS_CC);
+		php_info_print_module(module);
 	}
 	return ZEND_HASH_APPLY_KEEP;
 }
@@ -192,102 +186,94 @@ static int _display_module_info_def(zend_module_entry *module TSRMLS_DC) /* {{{
 
 /* {{{ php_print_gpcse_array
  */
-static void php_print_gpcse_array(char *name, uint name_length TSRMLS_DC)
+static void php_print_gpcse_array(char *name, uint32_t name_length)
 {
-	zval **data, **tmp, tmp2;
-	char *string_key;
-	uint string_len;
-	ulong num_key;
+	zval *data, *tmp;
+	zend_string *string_key;
+	zend_ulong num_key;
+	zend_string *key;
 
-	zend_is_auto_global(name, name_length TSRMLS_CC);
+	key = zend_string_init(name, name_length, 0);
+	zend_is_auto_global(key);
 
-	if (zend_hash_find(&EG(symbol_table), name, name_length+1, (void **) &data)!=FAILURE
-		&& (Z_TYPE_PP(data)==IS_ARRAY)) {
-		zend_hash_internal_pointer_reset(Z_ARRVAL_PP(data));
-		while (zend_hash_get_current_data(Z_ARRVAL_PP(data), (void **) &tmp) == SUCCESS) {
+	if ((data = zend_hash_find(&EG(symbol_table), key)) != NULL && (Z_TYPE_P(data) == IS_ARRAY)) {
+		ZEND_HASH_FOREACH_KEY_VAL(Z_ARRVAL_P(data), num_key, string_key, tmp) {
 			if (!sapi_module.phpinfo_as_text) {
 				php_info_print("<tr>");
 				php_info_print("<td class=\"e\">");
 			}
 
+			php_info_print("$");
 			php_info_print(name);
-			php_info_print("[\"");
+			php_info_print("['");
 
-			switch (zend_hash_get_current_key_ex(Z_ARRVAL_PP(data), &string_key, &string_len, &num_key, 0, NULL)) {
-				case HASH_KEY_IS_STRING:
-					if (!sapi_module.phpinfo_as_text) {
-						php_info_print_html_esc(string_key, string_len-1);
-					} else {
-						php_info_print(string_key);
-					}
-					break;
-				case HASH_KEY_IS_LONG:
-					php_info_printf("%ld", num_key);
-					break;
+			if (string_key != NULL) {
+				if (!sapi_module.phpinfo_as_text) {
+					php_info_print_html_esc(ZSTR_VAL(string_key), ZSTR_LEN(string_key));
+				} else {
+					php_info_print(ZSTR_VAL(string_key));
+				}
+			} else {
+				php_info_printf(ZEND_ULONG_FMT, num_key);
 			}
-			php_info_print("\"]");
+			php_info_print("']");
 			if (!sapi_module.phpinfo_as_text) {
 				php_info_print("</td><td class=\"v\">");
 			} else {
 				php_info_print(" => ");
 			}
-			if (Z_TYPE_PP(tmp) == IS_ARRAY) {
+			if (Z_TYPE_P(tmp) == IS_ARRAY) {
 				if (!sapi_module.phpinfo_as_text) {
+					zend_string *str = zend_print_zval_r_to_str(tmp, 0);
 					php_info_print("<pre>");
-					zend_print_zval_r_ex((zend_write_func_t) php_info_print_html_esc, *tmp, 0 TSRMLS_CC);
+					php_info_print_html_esc(ZSTR_VAL(str), ZSTR_LEN(str));
 					php_info_print("</pre>");
+					zend_string_release_ex(str, 0);
 				} else {
-					zend_print_zval_r(*tmp, 0 TSRMLS_CC);
+					zend_print_zval_r(tmp, 0);
 				}
 			} else {
-				tmp2 = **tmp;
-				if (Z_TYPE_PP(tmp) != IS_STRING) {
-					tmp = NULL;
-					zval_copy_ctor(&tmp2);
-					convert_to_string(&tmp2);
-				}
+				zend_string *tmp2;
+				zend_string *str = zval_get_tmp_string(tmp, &tmp2);
 
 				if (!sapi_module.phpinfo_as_text) {
-					if (Z_STRLEN(tmp2) == 0) {
+					if (ZSTR_LEN(str) == 0) {
 						php_info_print("<i>no value</i>");
 					} else {
-						php_info_print_html_esc(Z_STRVAL(tmp2), Z_STRLEN(tmp2));
+						php_info_print_html_esc(ZSTR_VAL(str), ZSTR_LEN(str));
 					}
 				} else {
-					php_info_print(Z_STRVAL(tmp2));
+					php_info_print(ZSTR_VAL(str));
 				}
 
-				if (!tmp) {
-					zval_dtor(&tmp2);
-				}
+				zend_tmp_string_release(tmp2);
 			}
 			if (!sapi_module.phpinfo_as_text) {
 				php_info_print("</td></tr>\n");
 			} else {
 				php_info_print("\n");
 			}
-			zend_hash_move_forward(Z_ARRVAL_PP(data));
-		}
+		} ZEND_HASH_FOREACH_END();
 	}
+	zend_string_efree(key);
 }
 /* }}} */
 
 /* {{{ php_info_print_style
  */
-void php_info_print_style(TSRMLS_D)
+void php_info_print_style(void)
 {
 	php_info_printf("<style type=\"text/css\">\n");
-	php_info_print_css(TSRMLS_C);
+	php_info_print_css();
 	php_info_printf("</style>\n");
 }
 /* }}} */
 
 /* {{{ php_info_html_esc
  */
-PHPAPI char *php_info_html_esc(char *string TSRMLS_DC)
+PHPAPI zend_string *php_info_html_esc(char *string)
 {
-	size_t new_len;
-	return php_escape_html_entities((unsigned char *) string, strlen(string), &new_len, 0, ENT_QUOTES, NULL TSRMLS_CC);
+	return php_escape_html_entities((unsigned char *) string, strlen(string), 0, ENT_QUOTES, NULL);
 }
 /* }}} */
 
@@ -296,31 +282,16 @@ PHPAPI char *php_info_html_esc(char *string TSRMLS_DC)
 
 char* php_get_windows_name()
 {
-	OSVERSIONINFOEX osvi;
+	OSVERSIONINFOEX osvi = EG(windows_version_info);
 	SYSTEM_INFO si;
-	PGNSI pGNSI;
-	PGPI pGPI;
-	BOOL bOsVersionInfoEx;
 	DWORD dwType;
 	char *major = NULL, *sub = NULL, *retval;
 
 	ZeroMemory(&si, sizeof(SYSTEM_INFO));
-	ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
-	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
 
-	if (!(bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO *) &osvi))) {
-		return NULL;
-	}
-
-	pGNSI = (PGNSI) GetProcAddress(GetModuleHandle("kernel32.dll"), "GetNativeSystemInfo");
-	if(NULL != pGNSI) {
-		pGNSI(&si);
-	} else {
-		GetSystemInfo(&si);
-	}
-
-	if (VER_PLATFORM_WIN32_NT==osvi.dwPlatformId && osvi.dwMajorVersion > 4 ) {
+	GetNativeSystemInfo(&si);
 
+	if (VER_PLATFORM_WIN32_NT==osvi.dwPlatformId && osvi.dwMajorVersion >= 10) {
 		if (osvi.dwMajorVersion == 10) {
 			if( osvi.dwMinorVersion == 0 ) {
 				if( osvi.wProductType == VER_NT_WORKSTATION ) {
@@ -330,7 +301,7 @@ char* php_get_windows_name()
 				}
 			}
 		}
-
+	} else if (VER_PLATFORM_WIN32_NT==osvi.dwPlatformId && osvi.dwMajorVersion >= 6) {
 		if (osvi.dwMajorVersion == 6) {
 			if( osvi.dwMinorVersion == 0 ) {
 				if( osvi.wProductType == VER_NT_WORKSTATION ) {
@@ -338,8 +309,7 @@ char* php_get_windows_name()
 				} else {
 					major = "Windows Server 2008";
 				}
-			} else
-			if ( osvi.dwMinorVersion == 1 ) {
+			} else if ( osvi.dwMinorVersion == 1 ) {
 				if( osvi.wProductType == VER_NT_WORKSTATION )  {
 					major = "Windows 7";
 				} else {
@@ -347,6 +317,11 @@ char* php_get_windows_name()
 				}
 			} else if ( osvi.dwMinorVersion == 2 ) {
 				/* could be Windows 8/Windows Server 2012, could be Windows 8.1/Windows Server 2012 R2 */
+				/* XXX and one more X - the above comment is true if no manifest is used for two cases:
+					- if the PHP build doesn't use the correct manifest
+					- if PHP DLL loaded under some binary that doesn't use the correct manifest
+
+					So keep the handling here as is for now, even if we know 6.2 is win8 and nothing else, and think about an improvement. */
 				OSVERSIONINFOEX osvi81;
 				DWORDLONG dwlConditionMask = 0;
 				int op = VER_GREATER_EQUAL;
@@ -361,7 +336,7 @@ char* php_get_windows_name()
 				VER_SET_CONDITION(dwlConditionMask, VER_MINORVERSION, op);
 				VER_SET_CONDITION(dwlConditionMask, VER_SERVICEPACKMAJOR, op);
 
-				if (VerifyVersionInfo(&osvi81, 
+				if (VerifyVersionInfo(&osvi81,
 					VER_MAJORVERSION | VER_MINORVERSION | VER_SERVICEPACKMAJOR,
 					dwlConditionMask)) {
 					osvi.dwMinorVersion = 3; /* Windows 8.1/Windows Server 2012 R2 */
@@ -376,7 +351,7 @@ char* php_get_windows_name()
 					} else {
 						major = "Windows Server 2012";
 					}
-				} 
+				}
 			} else if (osvi.dwMinorVersion == 3) {
 				if( osvi.wProductType == VER_NT_WORKSTATION )  {
 					major = "Windows 8.1";
@@ -387,8 +362,8 @@ char* php_get_windows_name()
 				major = "Unknown Windows version";
 			}
 
-			pGPI = (PGPI) GetProcAddress(GetModuleHandle("kernel32.dll"), "GetProductInfo");
-			pGPI(6, 0, 0, 0, &dwType);
+			/* No return value check, as it can only fail if the input parameters are broken (which we manually supply) */
+			GetProductInfo(6, 0, 0, 0, &dwType);
 
 			switch (dwType) {
 				case PRODUCT_ULTIMATE:
@@ -633,78 +608,6 @@ char* php_get_windows_name()
 					break;
 			}
 		}
-
-		if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2 ) {
-			if (GetSystemMetrics(SM_SERVERR2))
-				major = "Windows Server 2003 R2";
-			else if (osvi.wSuiteMask==VER_SUITE_STORAGE_SERVER)
-				major = "Windows Storage Server 2003";
-			else if (osvi.wSuiteMask==VER_SUITE_WH_SERVER)
-				major = "Windows Home Server";
-			else if (osvi.wProductType == VER_NT_WORKSTATION &&
-				si.wProcessorArchitecture==PROCESSOR_ARCHITECTURE_AMD64) {
-				major = "Windows XP Professional x64 Edition";
-			} else {
-				major = "Windows Server 2003";
-			}
-
-			/* Test for the server type. */
-			if ( osvi.wProductType != VER_NT_WORKSTATION ) {
-				if ( si.wProcessorArchitecture==PROCESSOR_ARCHITECTURE_IA64 ) {
-					if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
-						sub = "Datacenter Edition for Itanium-based Systems";
-					else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
-						sub = "Enterprise Edition for Itanium-based Systems";
-				}
-
-				else if ( si.wProcessorArchitecture==PROCESSOR_ARCHITECTURE_AMD64 ) {
-					if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
-						sub = "Datacenter x64 Edition";
-					else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
-						sub = "Enterprise x64 Edition";
-					else sub = "Standard x64 Edition";
-				} else {
-					if ( osvi.wSuiteMask & VER_SUITE_COMPUTE_SERVER )
-						sub = "Compute Cluster Edition";
-					else if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
-						sub = "Datacenter Edition";
-					else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
-						sub = "Enterprise Edition";
-					else if ( osvi.wSuiteMask & VER_SUITE_BLADE )
-						sub = "Web Edition";
-					else sub = "Standard Edition";
-				}
-			}
-		}
-
-		if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1 )	{
-			major = "Windows XP";
-			if( osvi.wSuiteMask & VER_SUITE_PERSONAL ) {
-				sub = "Home Edition";
-			} else if (GetSystemMetrics(SM_MEDIACENTER)) {
-				sub = "Media Center Edition";
-			} else if (GetSystemMetrics(SM_STARTER)) {
-				sub = "Starter Edition";
-			} else if (GetSystemMetrics(SM_TABLETPC)) {
-				sub = "Tablet PC Edition";
-			} else {
-				sub = "Professional";
-			}
-		}
-
-		if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0 ) {
-			major = "Windows 2000";
-
-			if (osvi.wProductType == VER_NT_WORKSTATION ) {
-				sub = "Professional";
-			} else {
-				if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
-					sub = "Datacenter Server";
-				else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
-					sub = "Advanced Server";
-				else sub = "Server";
-			}
-		}
 	} else {
 		return NULL;
 	}
@@ -756,7 +659,7 @@ void php_get_windows_cpu(char *buf, int bufsize)
 
 /* {{{ php_get_uname
  */
-PHPAPI char *php_get_uname(char mode)
+PHPAPI zend_string *php_get_uname(char mode)
 {
 	char *php_uname;
 	char tmp_uname[256];
@@ -796,16 +699,18 @@ PHPAPI char *php_get_uname(char mode)
 		char *winver = php_get_windows_name();
 		char wincpu[20];
 
+		ZEND_ASSERT(winver != NULL);
+
 		php_get_windows_cpu(wincpu, sizeof(wincpu));
 		dwBuild = (DWORD)(HIWORD(dwVersion));
-		
+
 		/* Windows "version" 6.2 could be Windows 8/Windows Server 2012, but also Windows 8.1/Windows Server 2012 R2 */
 		if (dwWindowsMajorVersion == 6 && dwWindowsMinorVersion == 2) {
 			if (strncmp(winver, "Windows 8.1", 11) == 0 || strncmp(winver, "Windows Server 2012 R2", 22) == 0) {
 				dwWindowsMinorVersion = 3;
 			}
 		}
-		
+
 		snprintf(tmp_uname, sizeof(tmp_uname), "%s %s %d.%d build %d (%s) %s",
 				 "Windows NT", ComputerName,
 				 dwWindowsMajorVersion, dwWindowsMinorVersion, dwBuild, winver?winver:"unknown", wincpu);
@@ -820,30 +725,6 @@ PHPAPI char *php_get_uname(char mode)
 	if (uname((struct utsname *)&buf) == -1) {
 		php_uname = PHP_UNAME;
 	} else {
-#ifdef NETWARE
-		if (mode == 's') {
-			php_uname = buf.sysname;
-		} else if (mode == 'r') {
-			snprintf(tmp_uname, sizeof(tmp_uname), "%d.%d.%d",
-					 buf.netware_major, buf.netware_minor, buf.netware_revision);
-			php_uname = tmp_uname;
-		} else if (mode == 'n') {
-			php_uname = buf.servername;
-		} else if (mode == 'v') {
-			snprintf(tmp_uname, sizeof(tmp_uname), "libc-%d.%d.%d #%d",
-					 buf.libmajor, buf.libminor, buf.librevision, buf.libthreshold);
-			php_uname = tmp_uname;
-		} else if (mode == 'm') {
-			php_uname = buf.machine;
-		} else { /* assume mode == 'a' */
-			snprintf(tmp_uname, sizeof(tmp_uname), "%s %s %d.%d.%d libc-%d.%d.%d #%d %s",
-					 buf.sysname, buf.servername,
-					 buf.netware_major, buf.netware_minor, buf.netware_revision,
-					 buf.libmajor, buf.libminor, buf.librevision, buf.libthreshold,
-					 buf.machine);
-			php_uname = tmp_uname;
-		}
-#else
 		if (mode == 's') {
 			php_uname = buf.sysname;
 		} else if (mode == 'r') {
@@ -860,25 +741,24 @@ PHPAPI char *php_get_uname(char mode)
 					 buf.machine);
 			php_uname = tmp_uname;
 		}
-#endif /* NETWARE */
 	}
 #else
 	php_uname = PHP_UNAME;
 #endif
 #endif
-	return estrdup(php_uname);
+	return zend_string_init(php_uname, strlen(php_uname), 0);
 }
 /* }}} */
 
 /* {{{ php_print_info_htmlhead
  */
-PHPAPI void php_print_info_htmlhead(TSRMLS_D)
+PHPAPI void php_print_info_htmlhead(void)
 {
 	php_info_print("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"DTD/xhtml1-transitional.dtd\">\n");
 	php_info_print("<html xmlns=\"http://www.w3.org/1999/xhtml\">");
 	php_info_print("<head>\n");
-	php_info_print_style(TSRMLS_C);
-	php_info_print("<title>phpinfo()</title>");
+	php_info_print_style();
+	php_info_printf("<title>PHP %s - phpinfo()</title>", PHP_VERSION);
 	php_info_print("<meta name=\"ROBOTS\" content=\"NOINDEX,NOFOLLOW,NOARCHIVE\" />");
 	php_info_print("</head>\n");
 	php_info_print("<body><div class=\"center\">\n");
@@ -886,25 +766,25 @@ PHPAPI void php_print_info_htmlhead(TSRMLS_D)
 /* }}} */
 
 /* {{{ module_name_cmp */
-static int module_name_cmp(const void *a, const void *b TSRMLS_DC)
+static int module_name_cmp(const void *a, const void *b)
 {
-	Bucket *f = *((Bucket **) a);
-	Bucket *s = *((Bucket **) b);
+	Bucket *f = (Bucket *) a;
+	Bucket *s = (Bucket *) b;
 
-	return strcasecmp(((zend_module_entry *)f->pData)->name,
-				  ((zend_module_entry *)s->pData)->name);
+	return strcasecmp(((zend_module_entry *)Z_PTR(f->val))->name,
+				  ((zend_module_entry *)Z_PTR(s->val))->name);
 }
 /* }}} */
 
 /* {{{ php_print_info
  */
-PHPAPI void php_print_info(int flag TSRMLS_DC)
+PHPAPI void php_print_info(int flag)
 {
 	char **env, *tmp1, *tmp2;
-	char *php_uname;
+	zend_string *php_uname;
 
 	if (!sapi_module.phpinfo_as_text) {
-		php_print_info_htmlhead(TSRMLS_C);
+		php_print_info_htmlhead();
 	} else {
 		php_info_print("phpinfo()\n");
 	}
@@ -941,8 +821,8 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 		}
 		php_info_print_box_end();
 		php_info_print_table_start();
-		php_info_print_table_row(2, "System", php_uname );
-		php_info_print_table_row(2, "Build Date", __DATE__ " " __TIME__ );
+		php_info_print_table_row(2, "System", ZSTR_VAL(php_uname));
+		php_info_print_table_row(2, "Build Date", __DATE__ " " __TIME__);
 #ifdef COMPILER
 		php_info_print_table_row(2, "Compiler", COMPILER);
 #endif
@@ -985,6 +865,7 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 
 #ifdef ZTS
 		php_info_print_table_row(2, "Thread Safety", "enabled" );
+		php_info_print_table_row(2, "Thread API", tsrm_api_name() );
 #else
 		php_info_print_table_row(2, "Thread Safety", "disabled" );
 #endif
@@ -995,10 +876,10 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 		php_info_print_table_row(2, "Zend Signal Handling", "disabled" );
 #endif
 
-		php_info_print_table_row(2, "Zend Memory Manager", is_zend_mm(TSRMLS_C) ? "enabled" : "disabled" );
+		php_info_print_table_row(2, "Zend Memory Manager", is_zend_mm() ? "enabled" : "disabled" );
 
 		{
-			const zend_multibyte_functions *functions = zend_multibyte_get_functions(TSRMLS_C);
+			const zend_multibyte_functions *functions = zend_multibyte_get_functions();
 			char *descr;
 			if (functions) {
 				spprintf(&descr, 0, "provided by %s", functions->provider_name);
@@ -1016,14 +897,14 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 #endif
 
 #if HAVE_DTRACE
-		php_info_print_table_row(2, "DTrace Support", "enabled" );
+		php_info_print_table_row(2, "DTrace Support", (zend_dtrace_enabled ? "enabled" : "available, disabled"));
 #else
 		php_info_print_table_row(2, "DTrace Support", "disabled" );
 #endif
 
-		php_info_print_stream_hash("PHP Streams",  php_stream_get_url_stream_wrappers_hash() TSRMLS_CC);
-		php_info_print_stream_hash("Stream Socket Transports", php_stream_xport_get_hash() TSRMLS_CC);
-		php_info_print_stream_hash("Stream Filters", php_get_stream_filters_hash() TSRMLS_CC);
+		php_info_print_stream_hash("PHP Streams",  php_stream_get_url_stream_wrappers_hash());
+		php_info_print_stream_hash("Stream Socket Transports", php_stream_xport_get_hash());
+		php_info_print_stream_hash("Stream Filters", php_get_stream_filters_hash());
 
 		php_info_print_table_end();
 
@@ -1038,13 +919,13 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 		if (sapi_module.phpinfo_as_text) {
 			php_info_print(zend_version);
 		} else {
-			zend_html_puts(zend_version, strlen(zend_version) TSRMLS_CC);
+			zend_html_puts(zend_version, strlen(zend_version));
 		}
 		php_info_print_box_end();
-		efree(php_uname);
+		zend_string_free(php_uname);
 	}
 
-	zend_ini_sort_entries(TSRMLS_C);
+	zend_ini_sort_entries();
 
 	if (flag & PHP_INFO_CONFIGURATION) {
 		php_info_print_hr();
@@ -1061,18 +942,17 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 
 	if (flag & PHP_INFO_MODULES) {
 		HashTable sorted_registry;
-		zend_module_entry tmp;
 
 		zend_hash_init(&sorted_registry, zend_hash_num_elements(&module_registry), NULL, NULL, 1);
-		zend_hash_copy(&sorted_registry, &module_registry, NULL, &tmp, sizeof(zend_module_entry));
-		zend_hash_sort(&sorted_registry, zend_qsort, module_name_cmp, 0 TSRMLS_CC);
+		zend_hash_copy(&sorted_registry, &module_registry, NULL);
+		zend_hash_sort(&sorted_registry, module_name_cmp, 0);
 
-		zend_hash_apply(&sorted_registry, (apply_func_t) _display_module_info_func TSRMLS_CC);
+		zend_hash_apply(&sorted_registry, _display_module_info_func);
 
 		SECTION("Additional Modules");
 		php_info_print_table_start();
 		php_info_print_table_header(1, "Module Name");
-		zend_hash_apply(&sorted_registry, (apply_func_t) _display_module_info_def TSRMLS_CC);
+		zend_hash_apply(&sorted_registry, _display_module_info_def);
 		php_info_print_table_end();
 
 		zend_hash_destroy(&sorted_registry);
@@ -1097,38 +977,38 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 	}
 
 	if (flag & PHP_INFO_VARIABLES) {
-		zval **data;
+		zval *data;
 
 		SECTION("PHP Variables");
 
 		php_info_print_table_start();
 		php_info_print_table_header(2, "Variable", "Value");
-		if (zend_hash_find(&EG(symbol_table), "PHP_SELF", sizeof("PHP_SELF"), (void **) &data) != FAILURE && Z_TYPE_PP(data) == IS_STRING) {
-			php_info_print_table_row(2, "PHP_SELF", Z_STRVAL_PP(data));
+		if ((data = zend_hash_str_find(&EG(symbol_table), "PHP_SELF", sizeof("PHP_SELF")-1)) != NULL && Z_TYPE_P(data) == IS_STRING) {
+			php_info_print_table_row(2, "PHP_SELF", Z_STRVAL_P(data));
 		}
-		if (zend_hash_find(&EG(symbol_table), "PHP_AUTH_TYPE", sizeof("PHP_AUTH_TYPE"), (void **) &data) != FAILURE && Z_TYPE_PP(data) == IS_STRING) {
-			php_info_print_table_row(2, "PHP_AUTH_TYPE", Z_STRVAL_PP(data));
+		if ((data = zend_hash_str_find(&EG(symbol_table), "PHP_AUTH_TYPE", sizeof("PHP_AUTH_TYPE")-1)) != NULL && Z_TYPE_P(data) == IS_STRING) {
+			php_info_print_table_row(2, "PHP_AUTH_TYPE", Z_STRVAL_P(data));
 		}
-		if (zend_hash_find(&EG(symbol_table), "PHP_AUTH_USER", sizeof("PHP_AUTH_USER"), (void **) &data) != FAILURE && Z_TYPE_PP(data) == IS_STRING) {
-			php_info_print_table_row(2, "PHP_AUTH_USER", Z_STRVAL_PP(data));
+		if ((data = zend_hash_str_find(&EG(symbol_table), "PHP_AUTH_USER", sizeof("PHP_AUTH_USER")-1)) != NULL && Z_TYPE_P(data) == IS_STRING) {
+			php_info_print_table_row(2, "PHP_AUTH_USER", Z_STRVAL_P(data));
 		}
-		if (zend_hash_find(&EG(symbol_table), "PHP_AUTH_PW", sizeof("PHP_AUTH_PW"), (void **) &data) != FAILURE && Z_TYPE_PP(data) == IS_STRING) {
-			php_info_print_table_row(2, "PHP_AUTH_PW", Z_STRVAL_PP(data));
+		if ((data = zend_hash_str_find(&EG(symbol_table), "PHP_AUTH_PW", sizeof("PHP_AUTH_PW")-1)) != NULL && Z_TYPE_P(data) == IS_STRING) {
+			php_info_print_table_row(2, "PHP_AUTH_PW", Z_STRVAL_P(data));
 		}
-		php_print_gpcse_array(ZEND_STRL("_REQUEST") TSRMLS_CC);
-		php_print_gpcse_array(ZEND_STRL("_GET") TSRMLS_CC);
-		php_print_gpcse_array(ZEND_STRL("_POST") TSRMLS_CC);
-		php_print_gpcse_array(ZEND_STRL("_FILES") TSRMLS_CC);
-		php_print_gpcse_array(ZEND_STRL("_COOKIE") TSRMLS_CC);
-		php_print_gpcse_array(ZEND_STRL("_SERVER") TSRMLS_CC);
-		php_print_gpcse_array(ZEND_STRL("_ENV") TSRMLS_CC);
+		php_print_gpcse_array(ZEND_STRL("_REQUEST"));
+		php_print_gpcse_array(ZEND_STRL("_GET"));
+		php_print_gpcse_array(ZEND_STRL("_POST"));
+		php_print_gpcse_array(ZEND_STRL("_FILES"));
+		php_print_gpcse_array(ZEND_STRL("_COOKIE"));
+		php_print_gpcse_array(ZEND_STRL("_SERVER"));
+		php_print_gpcse_array(ZEND_STRL("_ENV"));
 		php_info_print_table_end();
 	}
 
 
 	if ((flag & PHP_INFO_CREDITS) && !sapi_module.phpinfo_as_text) {
 		php_info_print_hr();
-		php_print_credits(PHP_CREDITS_ALL & ~PHP_CREDITS_FULLPAGE TSRMLS_CC);
+		php_print_credits(PHP_CREDITS_ALL & ~PHP_CREDITS_FULLPAGE);
 	}
 
 	if (flag & PHP_INFO_LICENSE) {
@@ -1233,7 +1113,7 @@ PHPAPI void php_info_print_table_colspan_header(int num_cols, char *header) /* {
 	if (!sapi_module.phpinfo_as_text) {
 		php_info_printf("<tr class=\"h\"><th colspan=\"%d\">%s</th></tr>\n", num_cols, header );
 	} else {
-		spaces = (74 - strlen(header));
+		spaces = (int)(74 - strlen(header));
 		php_info_printf("%*s%s%*s\n", (int)(spaces/2), " ", header, (int)(spaces/2), " ");
 	}
 }
@@ -1375,16 +1255,17 @@ void register_phpinfo_constants(INIT_FUNC_ARGS)
    Output a page of useful information about PHP and the current request */
 PHP_FUNCTION(phpinfo)
 {
-	long flag = PHP_INFO_ALL;
+	zend_long flag = PHP_INFO_ALL;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &flag) == FAILURE) {
-		return;
-	}
+	ZEND_PARSE_PARAMETERS_START(0, 1)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_LONG(flag)
+	ZEND_PARSE_PARAMETERS_END();
 
 	/* Andale!  Andale!  Yee-Hah! */
-	php_output_start_default(TSRMLS_C);
-	php_print_info(flag TSRMLS_CC);
-	php_output_end(TSRMLS_C);
+	php_output_start_default();
+	php_print_info((int)flag);
+	php_output_end();
 
 	RETURN_TRUE;
 }
@@ -1396,21 +1277,22 @@ PHP_FUNCTION(phpinfo)
 PHP_FUNCTION(phpversion)
 {
 	char *ext_name = NULL;
-	int ext_name_len = 0;
+	size_t ext_name_len = 0;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s", &ext_name, &ext_name_len) == FAILURE) {
-		return;
-	}
+	ZEND_PARSE_PARAMETERS_START(0, 1)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_STRING(ext_name, ext_name_len)
+	ZEND_PARSE_PARAMETERS_END();
 
 	if (!ext_name) {
-		RETURN_STRING(PHP_VERSION, 1);
+		RETURN_STRING(PHP_VERSION);
 	} else {
 		const char *version;
 		version = zend_get_module_version(ext_name);
 		if (version == NULL) {
 			RETURN_FALSE;
 		}
-		RETURN_STRING(version, 1);
+		RETURN_STRING(version);
 	}
 }
 /* }}} */
@@ -1419,13 +1301,14 @@ PHP_FUNCTION(phpversion)
    Prints the list of people who've contributed to the PHP project */
 PHP_FUNCTION(phpcredits)
 {
-	long flag = PHP_CREDITS_ALL;
+	zend_long flag = PHP_CREDITS_ALL;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &flag) == FAILURE) {
-		return;
-	}
+	ZEND_PARSE_PARAMETERS_START(0, 1)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_LONG(flag)
+	ZEND_PARSE_PARAMETERS_END();
 
-	php_print_credits(flag TSRMLS_CC);
+	php_print_credits((int)flag);
 	RETURN_TRUE;
 }
 /* }}} */
@@ -1439,7 +1322,7 @@ PHP_FUNCTION(php_sapi_name)
 	}
 
 	if (sapi_module.name) {
-		RETURN_STRING(sapi_module.name, 1);
+		RETURN_STRING(sapi_module.name);
 	} else {
 		RETURN_FALSE;
 	}
@@ -1452,12 +1335,14 @@ PHP_FUNCTION(php_sapi_name)
 PHP_FUNCTION(php_uname)
 {
 	char *mode = "a";
-	int modelen = sizeof("a")-1;
+	size_t modelen = sizeof("a")-1;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s", &mode, &modelen) == FAILURE) {
-		return;
-	}
-	RETURN_STRING(php_get_uname(*mode), 0);
+	ZEND_PARSE_PARAMETERS_START(0, 1)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_STRING(mode, modelen)
+	ZEND_PARSE_PARAMETERS_END();
+
+	RETURN_STR(php_get_uname(*mode));
 }
 
 /* }}} */
@@ -1470,8 +1355,8 @@ PHP_FUNCTION(php_ini_scanned_files)
 		return;
 	}
 
-	if (strlen(PHP_CONFIG_FILE_SCAN_DIR) && php_ini_scanned_files) {
-		RETURN_STRING(php_ini_scanned_files, 1);
+	if (php_ini_scanned_files) {
+		RETURN_STRING(php_ini_scanned_files);
 	} else {
 		RETURN_FALSE;
 	}
@@ -1487,7 +1372,7 @@ PHP_FUNCTION(php_ini_loaded_file)
 	}
 
 	if (php_ini_opened_path) {
-		RETURN_STRING(php_ini_opened_path, 1);
+		RETURN_STRING(php_ini_opened_path);
 	} else {
 		RETURN_FALSE;
 	}
